package net.pricefx.common.apibuilder.calculationflowaction;

import net.pricefx.common.apibuilder.Invocations;
import net.pricefx.common.dto.ChainedJobInfo;

import java.util.List;

/**
 * This action starts calculation of selected simulation. It's a programmatic version
 * of the same action available in UI. As such it allows user to schedule the
 * action in calculation flow formula.
 */
public class SimulationAction<ATTACHEDTO extends Help> extends Action {

	private String simulationLabel;
	private Boolean calculate;
	private int priority;
	private int distributedCalculationThreshold;
	private ATTACHEDTO backLink;
	
	public SimulationAction(ATTACHEDTO backLink, Invocations<MethodInvocationRecord> invocations, String simulationLabel) {
		super(invocations);
		this.backLink = backLink;
		this.simulationLabel = simulationLabel;
		this.calculate = false;
	}

	/**
	 * Whether the action is valid and should be triggered or not.
	 * 
	 * @param calculate
	 *            <ul>
	 *            <li>true - calculation of the simulation is started as
	 *            background process</li>
	 *            <li>false - calculation of the simulation is NOT started at all</li>
	 *            </ul>
	 */
	public SimulationAction<ATTACHEDTO> setCalculate(Boolean calculate) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".setCalculate", calculate));
		this.calculate = Boolean.TRUE.equals(calculate) ? true : false;
		return this;
	}
	
	/**
	 * Sets the prority of this job. 
	 * 
	 * @param priority Default is 0 (lowest user-definable priority). Max value is 9.
	 * @return
	 */
	public SimulationAction<ATTACHEDTO> setPriority(int priority) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".setPriority", priority));
		this.priority = priority;
		return this;		
	}	
	
	/**
	 * Sets a job-individual threshold to start a distributed calculation.
	 * Please note: Any values below 1000 may very well be counter-productive and make the job slower!
	 * 
	 * @param threshold The number of items that need to be part of the job in order to execute it in distributed mode.
	 * @return
	 */
	public SimulationAction<ATTACHEDTO> setDistributedCalculationThreshold(int threshold) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".setDistributedCalculationThreshold", threshold));
		this.distributedCalculationThreshold = threshold;
		return this;		
	}	
	
	public ATTACHEDTO back() {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".back"));
		return backLink;
	}

	/**
	 * Defines subsequent follow-on jobs
	 *
	 * @param jobs List of ChainedJobInfo objects
	 * @return
	 */
	public SimulationAction<ATTACHEDTO> setNextJobs(List<ChainedJobInfo> jobs) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".setNextJobs", jobs));
		this.jobs = jobs;
		return this;
	}

	@Override
	public String getObjectName() {
		return backLink.getObjectName() + ".SimulationAction";
	}

	public String getSimulationLabel() { return simulationLabel; }
	public Boolean getCalculate() {	return calculate; }
	public int getPriority() { return priority; }
	public int getDistributedCalculationThreshold() {	return distributedCalculationThreshold; }
	public List<ChainedJobInfo> getNextJobs() { return jobs; }
	
	@Override public String toString() {
		return "SimulationAction [simulationLabel=" + simulationLabel + ", calculate=" + calculate + ", backLink=" + backLink
				+ ", toString()=" + super.toString() + "]";
	}
	
}