package net.pricefx.formulaengine;

import java.util.List;
import java.util.Map;

import net.pricefx.domain.RebateRecord;
import net.pricefx.server.dto.calculation.ResultMatrix;
import net.pricefx.server.dto.calculation.ResultRebateRecord;

/**
 * The RebateRecordManager API is exposed to formula groovy elements in two separate contexts:
 * 		1) in a RebateAgreement calculation (typeCode='RBA' aka RA), when the formula context's 'currentItem' would be an individual line item (typeCode='RBLI' aka RLI)
 * 		2) in a RebateAgreement 'read-only' evaluation (when opening an uneditable RA);  currentItem is again a RLI
 * 		3) in a RebateRecordSet calculation, the formula context's 'currentItem' would be an individual RebateRecord (typeCode='RR')
 * The API can be accessed via the 'rebateRecords' Groovy binding.
 *
 * Not all methods are implemented in both contexts:  RRs can only be added in RA calculations.(1).  Similarly setRank() is only effective in context 1.
 * Context 2 obviously does not allow any modification, be it adding or updating RRs.
 * Context 3 only allows updaitng existing RRs (ithrough the set(..) mehtods).
 */
public interface IRebateRecordManager {

	/**
	 * Add a RR to the current RA.lineitem with the default name and optional initial generic attribute values
	 */
	public RebateRecord add(Map<?,?>... attributeValues);

	/**
	 * Add a RR to the current RA.lineitem with the given name and optional initial generic attribute values
	 */
	public RebateRecord add(String rebateRecordName, Map<?,?>... attributeValues);
	
	/**
	 * Add a RR in the given RR-set to the current RA.lineitem with the given name and optional initial generic attribute values
	 * If the RR-set with the given label does not yet exist, it will be created as a side-effect.
	 */
	public RebateRecord add(String rebateRecordName, String rebateRecordSetLabel, Map<?,?>... attributeValues);
	
	/**
	 * Sets the rank on the default RR.  RRs with a lower rank are calculated first within a given agreement. 
	 */
	public boolean setRank(int rank);
	
	/**
	 * Sets the rank on the named RR.  RRs with a lower rank are calculated first within a given agreement. 
	 */
	public boolean setRank(String rebateRecordName, int rank);

	/**
	 * Get all RRs linked to the the current agreement (via RR --&gt; RLI --&gt; RBA).
	 */
	public List<RebateRecord> getAll();		
	
	/**
	 * Get all RRs linked to the line item with the given lineID in the current agreement (via RR --&gt; RLI).
	 */
	public List<RebateRecord> getAll(String lineId);

	/**
	 * Get the RR with the default name linked to the current RA.lineitem
	 */
	public RebateRecord get();

	/**
	 * Get the RR with the given name linked to the current RA.lineitem
	 */
	public RebateRecord get(String rebateRecordName);

	/**
	 * In the given RR-set, get the RR with the given name linked to the current RA.lineitem
	 */
	public RebateRecord get(String rebateRecordName, String rebateRecordSetLabel);

	/**
	 * Update genetic attributes of the RR with the default name (linked to the current RA.lineitem)
	 */
	public boolean set(Map<?,?>... attributeValues);

	/**
	 * Update genetic attributes of the RR with the given name (linked to the current RA.lineitem)
	 */
	public boolean set(String rebateRecordName, Map<?,?>... attributeValues);
	
	/**
	 * In the given RR-set, update genetic attributes of the RR with the given name (linked to the current RA.lineitem)
	 */
	public boolean set(String rebateRecordName, String rebateRecordSetLabel, Map<?,?>... attributeValues);

	/**
	 * Get the formula element result for the RR with the default name linked to the current RA.lineitem
	 */
	public Object getResult(String elementName);

	/**
	 * Get the formula element result for the RR with the given name linked to the current RA.lineitem
	 */
	public Object getResult(String rebateRecordName, String elementName);

	/**
	 * In the given RR-set, get the formula element result for the RR with the given name linked to the current RA.lineitem
	 */
	public Object getResult(String rebateRecordName, String rebateRecordSetLabel, String elementName);

	/**
	 * Return a clickable link to the RR with the default name, as the result of a formula element
	 */
	public ResultRebateRecord asLink();

	/**
	 * Return a clickable link to the RR with the given name, as the result of a formula element
	 */
	public ResultRebateRecord asLink(String rebateRecordName);

	/**
	 * Return a clickable link to the RR for the given name and RR-set, as the result of a formula element
	 */
	public ResultRebateRecord asLink(String rebateRecordName, String rebateRecordSetLabel);
	/**
	 * Return a ResultMatrix listing all RRs in the agreement, optionally the fields/attributes to be shown as matrix columns.
	 */
	public ResultMatrix asResultMatrix(final String... attributes);

	@Deprecated
	public Object buildDataTableChart(String datamartName, String dateDimFieldName);
	
}