package net.pricefx.logic.core;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import net.pricefx.logic.core.model.AbstractFormula;
import net.pricefx.logic.core.model.AbstractFormulaElement;
import net.pricefx.logic.core.model.ContextParameter;
import net.pricefx.logic.core.model.FormulaErrorElementResult;
import net.pricefx.logic.core.model.PricingFormula;
import net.pricefx.logic.core.model.WorkflowFormula;
import net.pricefx.pckg.filesystem.FileSystemOpsBase;
import net.pricefx.pckg.processing.ProcessingContextImpl;
import net.pricefx.pckg.transform.TransformCalculationLogic;
import net.pricefx.pckg.transform.TransformWorkflowFormula;
import net.pricefx.restapiclient.PfxClientException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;

public class LogicSerializer extends AbstractLogicSerializer<Path> {

	private static final Logger log = LoggerFactory.getLogger(LogicSerializer.class);

	public LogicSerializer(AbstractFormula onDiskLogic) {
		super(onDiskLogic);
	}

	public static Path getFetchDir(String logicName) {
		return Paths.get(TransformCalculationLogic.DIRNAME, logicName);
	}

	public static Path getLogicJsonPath(Path logicPath) {
		return logicPath.resolve(LOGIC_JSON_FILENAME);
	}

	public static Path getElementsDirPath(Path logicDir) {
		return logicDir.resolve(ELEMENTS_SUBDIR);
	}

	public static Path getResultsDirPath(Path logicDir) {
		return logicDir.resolve(RESULTS_SUBDIR);
	}

	public static Path getElementPath(Path logicDir, AbstractFormulaElement element) {
		if (element == null) {
			return null;
		}
		return getElementsDirPath(logicDir).resolve(getFileNameForElement(element));
	}

	public Path getElementPath(AbstractFormulaElement element) {
		return getElementPath(logicDir, element);
	}

	public Path getParamsDirPath(Path logicDir) {
		return logicDir.resolve(PARAMETERS_SUBDIR);
	}

	public Path getParamsJson(Path logicDir, String paramsFileName) {
		return getParamsDirPath(logicDir).resolve(paramsFileName + PARAM_FILE_SUFFIX);
	}

	public static Path getResultJsonPath(Path logicDir, String presetName) {
		return getResultsDirPath(logicDir).resolve(presetName + PARAM_FILE_SUFFIX);
	}

	@Override
	public void writeElementAsFile(AbstractFormulaElement element) throws IOException {
		// Dump element
		Path elementFile = getElementPath(element);
		log.debug("(Over)writing logic element file {}", elementFile);

		if (element.getFormulaExpression() != null) {
			Files.write(elementFile, element.getFormulaExpression().getBytes("UTF8"));
		} else {
			log.warn("Element '{}' is null. Writing it as empty file {}.", element.getElementName(), elementFile);
			Files.write(elementFile, "".getBytes("UTF8"));
		}

		// TODO: Store all element properties to the disk (not only source code)
	}

	/**
	 * Dumps a given logic to logic.json on the disk
	 *
	 * @return Path to created logic.json file
	 */
	public void writeLogicJsonFile() throws IOException {
		writeCanonicalJson(new ProcessingContextImpl(), logicDir.resolve(LOGIC_JSON_FILENAME), onDiskLogic.getNode());
	}

	public <T> T readJsonFile(Path file, TypeReference type) throws IOException {
		return (T) new ObjectMapper().readValue(new FileSystemOpsBase().readText(file), type);
	}

	public List<ContextParameter> readParamsFile(String fileName) throws IOException {
		try {
			Path paramsFile = getParamsJson(logicDir, fileName);

			log.debug("Reading parameters " + paramsFile);

			return readJsonFile(paramsFile, new TypeReference<List<ContextParameter>>() {
			});
		} catch (IOException e) {
			String msg = String.format("Cannot read preset file %s", fileName);
			throw new IOException(msg, e);

		}
	}

	public <T extends ContextParameter> void writeParamsJson(List<T> parameters, String fileName) throws IOException {
		Path paramsDirectory = createDirectory(logicDir, PARAMETERS_SUBDIR);

		Path paramsFile = createFilePath(paramsDirectory, fileName + PARAM_FILE_SUFFIX);
		String content = Utils.serializeToJSONAsString(parameters);

		log.debug("Writing parameter file " + paramsFile);
		writeText(paramsFile, content);

	}

	public String[] readParamFiles() {
		try {
			List<Path> paths = list(getParamsDirPath(logicDir));

			String[] result = new String[paths.size()];

			for (int i = 0; i < paths.size(); i++) {
				result[i] = paths.get(i).getFileName().toString();
				if (result[i].endsWith(PARAM_FILE_SUFFIX)) {
					result[i] = result[i].substring(0, result[i].length() - PARAM_FILE_SUFFIX.length());
				}
			}
			return result;
		} catch (IOException e) {
			return new String[0];
		}
	}

	public void deleteParamsFile(String fileName) throws IOException {
		Path paramsFile = getParamsJson(logicDir, fileName);

		log.debug("Deleting " + paramsFile);

		delete(paramsFile);
	}


	public static List<FormulaErrorElementResult> readResultJson(Path logicFolder, String presetName) throws IOException {
		Path resultJsonFile = getResultJsonPath(logicFolder, presetName);

		log.debug("Reading " + resultJsonFile);
		return Utils.getObjectMapper().readValue(resultJsonFile.toFile(), new TypeReference<List<FormulaErrorElementResult>>() {});
	}

	public static Path getNewPath(Path modulePath, String businessKey, AbstractFormula logic) {
		try {

			if (logic instanceof PricingFormula) {
				return modulePath.resolve(TransformCalculationLogic.DIRNAME).resolve(businessKey);
			} else if (logic instanceof WorkflowFormula) {
				return modulePath.resolve(TransformWorkflowFormula.DIRNAME).resolve(businessKey);
			} else {
				throw new IllegalArgumentException("getConsumer: " + logic.getClass().getName() + " is not implemented");
			}

		} catch (IllegalArgumentException ex) {
			return null;
		}
	}
}
