package net.pricefx.server.dto.calculation;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Provides means for output elements' formatting.
 *
 * <p>
 * The usage is simple, instead of returning the output value, create a new {@code AttributedResult} instance like this:
 * {@code api.attributedResult(plain value)}, customize it by calling its various methods, and return it finally.
 *
 * <p>
 * Example:
 * <blockquote>
 * <pre>{@code
 * def cost = api.getElement("MarginPct")
 * def myOptions = ["A" , "B" , "C" ]
 * return api.attributedResult(cost)
 *         .withBackgroundColor(cost < 0.30 ? "red" : "#0101DF")
 *         .withSuffix(cost < 0.30 ? "!!!" : null)
 *         .withTextColor(cost < 0.30 ? "white" : null)
 *         .withTextDecoration(cost < 0.30 ? "underline" : null)
 *         .withManualOverrideValueOptions(myOptions)
 * }</pre>
 * </blockquote>
 */
public class AttributedResult {

	private final Object result;

	private final Map<String, Object> attrs;

	public AttributedResult(Object result) {
		this.result = result;
		this.attrs = new HashMap<>();
	}

	public AttributedResult(Object result, Map<String, Object> attrs) {
		this.result = result;
		this.attrs = new HashMap<>(attrs);
	}

	public Object getResult() {
		return result;
	}

	public Map<String, Object> getAttributes() {
		return Collections.unmodifiableMap(attrs);
	}

	@SuppressWarnings("unchecked")
	public <T> T getAttribute(String name) {
		return (T) attrs.get(name);
	}

	@SuppressWarnings("unchecked")
	public <T> T setAttribute(String name, Object value) {
		return (T) attrs.put(name, value);
	}

	@SuppressWarnings("unchecked")
	public <T> T clearAttribute(String name) {
		return (T) attrs.remove(name);
	}

	public boolean containsAttribute(String name) {
		return attrs.containsKey(name);
	}

	/**
	 * Adds any valid CSS style to the cell.
	 * Note: There is no check if the CSS is actually valid, the values are just passed through as-is
	 *
	 * @param css The full CSS style parameter in the form of "property:value;"
	 * @see <a href="http://www.w3schools.com/cssref/">CSS Reference</a>
	 */	
	public AttributedResult withRawCSS(String css) {
		if (css != null && !css.isEmpty()) {
			setAttribute("RAWCSS", css);
		}
		return this;
	}
	
	/**
	 * Sets a suffix to be appended to the result.
	 */
	public AttributedResult withSuffix(String suffix) {
		setAttribute("suffix", suffix);
		return this;
	}

	/**
	 * Sets the text (foreground) color.
	 *
	 * @see <a href="http://www.w3schools.com/cssref/css_colors_legal.asp">CSS Legal Color Values</a>
	 */
	public AttributedResult withTextColor(String cssColor) {
		if (cssColor != null && !cssColor.isEmpty()) {
			setAttribute("color", cssColor);
		}
		return this;
	}

	/**
	 * Sets the decoration to be added to the resulting text.
	 *
	 * <p>
	 * Valid options are {@code underline}, {@code line-through}, and {@code overline}.
	 * </p>
	 *
	 * @see <a href="http://www.w3schools.com/cssref/pr_text_text-decoration.asp">CSS text-decoration Property</a>
	 */
	public AttributedResult withTextDecoration(String textDecoration) {
		if ("overline".equalsIgnoreCase(textDecoration) || "line-through".equalsIgnoreCase(textDecoration)
				|| "underline".equalsIgnoreCase(textDecoration)) {
			setAttribute("text-decoration", textDecoration);
		}
		return this;
	}

	/**
	 * Sets the background color.
	 *
	 * @see <a href="http://www.w3schools.com/cssref/css_colors_legal.asp">CSS Legal Color Values</a>
	 */
	public AttributedResult withBackgroundColor(String cssColor) {
		if (cssColor != null && !cssColor.isEmpty()) {
			setAttribute("background-color", cssColor);
		}
		return this;
	}
	
	/**
	 * In case the result element is overridable, this method defines a set (list) of values tha the user can pick from (vs. default free-style data entry) 
	 * IMPORTANT NOTE: This is meant to list a few options only. Not a large list of choices! There is not a hard limitation here, but if this is used widely in a particular configuration
	 * and the list of options is long the browser may not be able to render it anymore and/or the data transfer speeds may suffer.
	 * 
	 * @param options The list of valid options in the UI
	 * @return
	 */
	public AttributedResult withManualOverrideValueOptions(List<Object> options) {
		setAttribute("overrideValueOptions", options);

		return this;
	}	
	
	/**
	 * In case the result element is overridable, this method defines if the selection box shows also the "empty value" option.
	 * Default is true
	 * @return
	 */
	public AttributedResult withManualOverrideAllowEmpty(Boolean allow) {
		setAttribute("overrideAllowEmpty", allow == null ? false : allow.booleanValue());

		return this;
	}	
}
